package com.gic.enterprise.web.controller.wm;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.web.vo.wm.WmStoreDetailVO;
import com.gic.enterprise.web.vo.wm.WmStoreVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.OperationResultUtils;
import com.gic.enterprise.constant.MallModeEnum;
import com.gic.enterprise.constant.WmStoreConfigTypeEnum;
import com.gic.enterprise.dto.WmStoreDTO;
import com.gic.enterprise.dto.wm.WmStoreConfigDTO;
import com.gic.enterprise.service.WmStoreApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;

@RestController
@RequestMapping("/wm-mall-store")
public class WmMallStoreController {
    private final static Logger LOGGER = LogManager.getLogger(WmMallStoreController.class);

    @Autowired
    private WmStoreApiService wmStoreApiService;

    @RequestMapping("/save-wm-store")
    public RestResponse save(WmStoreDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return OperationResultUtils.operationResult(wmStoreApiService.saveWmStore(dto),
                OperationResultUtils.LOG_NEW + "微盟商城店铺", dto.getWmPidName());
    }

    @RequestMapping("/edit-wm-store")
    public RestResponse edit(WmStoreDTO dto) {
        return OperationResultUtils.operationResult(wmStoreApiService.modifyWmStore(dto),
                OperationResultUtils.LOG_EDIT + "微盟商城店铺", dto.getWmPidName());
    }

    @RequestMapping("/get-wm-store-detail")
    public RestResponse getDetail(Integer wmMallStoreId) {
        return ResultControllerUtils.commonResultOne(wmStoreApiService.getWmStoreByWmMallStoreId(wmMallStoreId),
                WmStoreDetailVO.class);
    }

    @RequestMapping("/list-wm-store")
    public RestResponse listWmStore() {
        ServiceResponse<List<WmStoreDTO>> listResponse = wmStoreApiService.listWmStore(UserDetailUtils.getUserDetail().getEnterpriseId());
        if (listResponse.isSuccess()) {
            List<WmStoreDTO> list = listResponse.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                List<WmStoreVO> voList = list.stream()
                        .map(e -> {
                            WmStoreVO vo = EntityUtil.changeEntityNew(WmStoreVO.class, e);
                            ServiceResponse<Set<String>> configSet = wmStoreApiService.getWmStoreConfigByWmMainAccount(e.getWmMainAccount());
                            if (configSet.isSuccess()) {
                                Set<String> set = configSet.getResult();
                                if (set.contains(String.valueOf(WmStoreConfigTypeEnum.COUPON_CONFIG.getCode()))) {
                                    vo.setHasBindCoupon(1);
                                }
                                boolean isAllConfig = Stream.of(WmStoreConfigTypeEnum.values())
                                        .allMatch(configType -> set.contains(String.valueOf(configType.getCode())));
                                if (isAllConfig){
                                    vo.setHasCompleteConfig(1);
                                }
                            }
                            return vo;
                        })
                        .collect(Collectors.toList());
                return RestResponse.success(voList);
            }

        }
        return RestResponse.failure(listResponse.getCode(), listResponse.getMessage());
    }

    /**
     * 授权
     * @Title: authWmStore 
     * @Description:
     * @author guojuxing
     * @param wmMallStoreId 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("/auth-wm-store")
    public RestResponse authWmStore(Integer wmMallStoreId) {
        ServiceResponse<String> result = wmStoreApiService.authWmStore(wmMallStoreId);
        return OperationResultUtils.operationResult(result, "授权微盟商城店铺",
                OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/list-mall-mode")
    public RestResponse listMallMode() {
        Map<String, String> result = new HashMap<>(16);
        for (MallModeEnum modeEnum : MallModeEnum.values()) {
            result.put(String.valueOf(modeEnum.getCode()), modeEnum.getMsg());
        }
        return RestResponse.success(result);
    }

    @RequestMapping("/list-wm-store-config")
    public RestResponse listWmStoreConfig() {
        Map<String, String> result = new HashMap<>(16);
        for (WmStoreConfigTypeEnum modeEnum : WmStoreConfigTypeEnum.values()) {
            if (modeEnum.isHasConfigData()) {
                result.put(String.valueOf(modeEnum.getCode()), modeEnum.getMessage());
            }
        }
        return RestResponse.success(result);
    }

    @RequestMapping("/save-or-update-wm-store-config")
    public RestResponse saveOrUpdateWmStoreConfig(WmStoreConfigDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return OperationResultUtils
                .operationResult(wmStoreApiService.saveOrUpdateWmStoreConfig(dto),
                        OperationResultUtils.LOG_EDIT + "微盟商城-店铺配置"
                                + WmStoreConfigTypeEnum.getMessageByCode(dto.getStoreConfigType()),
                        dto.getWmMainAccount());
    }

    @RequestMapping("/get-wm-store-config-by-wm-main-account")
    public RestResponse getWmStoreConfigByWmMainAccount(String wmMainAccount, Integer wmStoreConfigType) {
        return ResultControllerUtils
                .commonResult(wmStoreApiService.getWmStoreConfigByWmMainAccount(wmMainAccount, wmStoreConfigType));
    }
}
